//! # Write-only arena, without deletion

use std::{marker::PhantomData, num::NonZeroU32};

use super::ptr::Ptr;

#[derive(Clone)]
pub struct Arena<T, P> {
    mem: Vec<T>,
    _ptr_ty: PhantomData<P>,
}

fn calc_idx(v: NonZeroU32) -> usize {
    v.get() as usize - 1
}

impl<T, P> Arena<T, P> {
    pub fn get_mut(&mut self, ptr: P) -> Option<&mut T>
    where
        P: Ptr,
    {
        self.mem.get_mut(calc_idx(ptr.into()))
    }

    pub fn get(&self, ptr: P) -> Option<&T>
    where
        P: Ptr,
    {
        self.mem.get(calc_idx(ptr.into()))
    }

    pub fn insert(&mut self, obj: T) -> P
    where
        P: Ptr,
    {
        let idx = self.mem.len();
        self.mem.push(obj);

        idx.checked_add(1)
            .and_then(|v| v.try_into().ok())
            .and_then(NonZeroU32::new)
            .expect("shit happens")
            .into()
    }
}

impl<T, P> Default for Arena<T, P> {
    fn default() -> Self {
        Self::new()
    }
}

impl<T, P> Arena<T, P> {
    pub const fn new() -> Self {
        Self {
            mem: Vec::new(),
            _ptr_ty: PhantomData,
        }
    }
}
